﻿'
' Copyright (c) 2021, MindFusion LLC - Bulgaria.
'

Imports System.Collections.ObjectModel
Imports System.ComponentModel


''' <summary>
''' Interaction logic for MainWindow.xaml
''' </summary>
Partial Public Class MainWindow
	Inherits Window
	Public Sub New()
		InitializeComponent()
	End Sub

	Private Sub OnLoaded(ByVal sender As Object, ByVal e As RoutedEventArgs)
        ' set large enough Bounds to fit 100000 nodes
        diagram.Bounds = New Rect(0, 0, 40 + 20 * 100000, 1520)

		' this size is used when data items do not expose Width and Height properties
		diagram.DefaultNodeSize = New Size(42, 16)

		' do not display arrowheads
		diagram.LinkHeadShape = Nothing

		' enable two way binding
		diagram.BindingMode = BindingMode.TwoWay

		' create data items when the user draws new nodes and links
		diagram.InstantiateNodeData = (Function(node) New NodeData() With { _
		 .Text = "new" _
		})
		diagram.InstantiateLinkData = (Function(link) New LinkData() With { _
		 .Text = "new" _
		})

		' allow editing items' text by double-click
		diagram.AllowInplaceEdit = True

		' create a collection of node data items
		Dim nodes = New ObservableCollection(Of NodeData)()
        For i As Integer = 0 To 99999
            Dim node = New NodeData()
            node.X = 10 + i * 20
            node.Y = 260 + 250 * Math.Sin(Math.PI * i / 22.5)
            node.Value = i
            node.Text = i.ToString()
            nodes.Add(node)
        Next

		' set the collection as data source
		diagram.NodesSource = nodes

		' create a collection of link data items
		Dim links = New ObservableCollection(Of LinkData)()
        For i As Integer = 0 To 50000 - 8
            Dim link = New LinkData()
            link.Origin = nodes(2 * i)
            link.Destination = nodes(2 * i + 13)
            link.Text = ""
            links.Add(link)
        Next

		' set the collection as data source
		diagram.LinksSource = links
	End Sub

	Private Sub OnNodeRealized(ByVal sender As Object, ByVal e As VirtualNodeEventArgs)
		Dim nodeData = DirectCast(e.DataItem, NodeData)
		e.Node.Brush = New SolidColorBrush(Color.FromArgb(255, 0, CByte(100 + nodeData.Value Mod 155), CByte(nodeData.Value Mod 255)))
	End Sub

	Private Sub OnLinkRealized(ByVal sender As Object, ByVal e As VirtualLinkEventArgs)
		e.Link.ZIndex = 0
	End Sub
End Class

Public Class NodeData
	Implements INotifyPropertyChanged
	Private m_x As Double
	Public Property X() As Double
		Get
			Return m_x
		End Get
		Set(ByVal value As Double)
			If m_x <> value Then
				m_x = value
				'Debug.WriteLine("x: " + x);
				RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs("X"))
			End If
		End Set
	End Property

	Private m_y As Double
	Public Property Y() As Double
		Get
			Return m_y
		End Get
		Set(ByVal value As Double)
			If m_y <> value Then
				m_y = value
				'Debug.WriteLine("y: " + y);
				RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs("Y"))
			End If
		End Set
	End Property

	Private m_text As String
	Public Property Text() As String
		Get
			Return m_text
		End Get
		Set(ByVal value As String)
			If m_text <> value Then
				m_text = value
				'Debug.WriteLine("text: " + text);
				RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs("Text"))
			End If
		End Set
	End Property

	Public Property Value() As Integer
		Get
			Return m_Value
		End Get
		Set(ByVal value As Integer)
			m_Value = value
		End Set
	End Property
	Private m_Value As Integer
	Public Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged
End Class

Public Class LinkData
	Implements INotifyPropertyChanged
	Private m_origin As NodeData
	Public Property Origin() As NodeData
		Get
			Return m_origin
		End Get
		Set(ByVal value As NodeData)
			If m_origin IsNot value Then
				m_origin = value
				RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs("Origin"))
			End If
		End Set
	End Property

	Private m_destination As NodeData
	Public Property Destination() As NodeData
		Get
			Return m_destination
		End Get
		Set(ByVal value As NodeData)
			If m_destination IsNot value Then
				m_destination = value
				RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs("Destination"))
			End If
		End Set
	End Property

	Public Property Text() As String
		Get
			Return m_Text
		End Get
		Set(ByVal value As String)
			m_Text = value
		End Set
	End Property
	Private m_Text As String

	Public Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged
End Class